/*
 * File name: GridCanvasUtil.java
 * Date:      2014/11/09 20:35
 * Author:    Jan Faigl
 */
package cz.cvut.fel.pr1;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author jf
 */
public class GridCanvasUtil {

    private GridCanvasUtil() {
    }

    /**
     * Bresenham's line algorithm to raster a straight-line segment into a grid
     * http://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm
     *
     * @param p0
     * @param p1
     * @return array of Coords representing rasterized segment from p0 to p1
     */
    public static Coords[] drawGridLine(Coords p0, Coords p1) {
        int x0 = p0.getX();
        int y0 = p0.getY();
        int x1 = p1.getX();
        int y1 = p1.getY();
        int sx = Math.abs(x0 - x1) + 1;
        int sy = Math.abs(y0 - y1) + 1;
        Coords[] ret = new Coords[sx > sy ? sx : sy];
        int i = 0;
        int dx = x1 - x0;
        int dy = y1 - y0;
        boolean steep = Math.abs(dy) >= Math.abs(dx);
        if (steep) {
            int t = x0;
            x0 = y0;
            y0 = t; //swap
            t = x1;
            x1 = y1;
            y1 = t; //swap
            dx = x1 - x0;
            dy = y1 - y0;
        }
        int xstep = 1;
        if (dx < 0) {
            xstep = -1;
            dx = -dx;
        }
        int ystep = 1;
        if (dy < 0) {
            ystep = -1;
            dy = -dy;
        }
        int twoDy = 2 * dy;
        int twoDyTwoDx = twoDy - 2 * dx; // 2*Dy - 2*Dx
        int e = twoDy - dx; //2*Dy - Dx
        int y = y0;
        int xDraw, yDraw;
        for (int x = x0; x != x1; x += xstep) {
            if (steep) {
                xDraw = y;
                yDraw = x;
            } else {
                xDraw = x;
                yDraw = y;
            }
            ret[i++] = p0.createCoords(xDraw, yDraw);
            // next
            if (e > 0) {
                e += twoDyTwoDx; //E += 2*Dy - 2*Dx;
                y = y + ystep;
            } else {
                e += twoDy; //E += 2*Dy;
            }
        }
        ret[i++] = p0.createCoords(x1, y1);
        return ret;
    }

    /**
     * Created rasterized representation of the given circle at the point (x,y)
     * and radius radius
     *
     * @param ct
     * @param radius
     * @return array of Points representing rasterized circle at (x,y) and with
     * the radius radius
     */
    public static Coords[] drawGridCircle(Coords ct, int radius) {
        int x = ct.getX();
        int y = ct.getY();
        Coords[] ret = null;
        List<Coords> circle = new ArrayList<>();
        int cx = 0;
        int cy = radius;
        int df = 1 - radius;
        int d_e = 3;
        int d_se = -2 * radius + 5;
        int xpcx, xmcx, xpcy, xmcy;
        int ypcy, ymcy, ypcx, ymcx;

        do {
            ypcy = y + cy;
            ymcy = y - cy;
            if (cx > 0) {
                xpcx = x + cx;
                xmcx = x - cx;
                circle.add(ct.createCoords(xmcx, ypcy));
                circle.add(ct.createCoords(xpcx, ypcy));
                circle.add(ct.createCoords(xmcx, ymcy));
                circle.add(ct.createCoords(xpcx, ymcy));
            } else {
                circle.add(ct.createCoords(x, ymcy));
                circle.add(ct.createCoords(x, ypcy));
            }
            xpcy = x + cy;
            xmcy = x - cy;
            if ((cx > 0) && (cx != cy)) {
                ypcx = y + cx;
                ymcx = y - cx;
                circle.add(ct.createCoords(xmcy, ypcx));
                circle.add(ct.createCoords(xpcy, ypcx));
                circle.add(ct.createCoords(xmcy, ymcx));
                circle.add(ct.createCoords(xpcy, ymcx));
            } else if (cx == 0) {
                circle.add(ct.createCoords(xmcy, y));
                circle.add(ct.createCoords(xpcy, y));
            }
            if (df < 0) {
                df += d_e;
                d_e += 2;
                d_se += 2;
            } else {
                df += d_se;
                d_e += 2;
                d_se += 4;
                cy--;
            }
            cx++;
        } while (cx <= cy);
        if (circle.size() > 0) {
            ret = new Coords[circle.size()];
            for (int i = 0; i < circle.size(); ++i) {
                ret[i] = circle.get(i);
            }
        }
        return ret;
    }

}
